<?php

/**
 * Lifestyle Webconsulting GmbH
 *
 * LICENSE: This Software is the property of Lifestyle Webconsulting GmbH (Aschaffenburg, Germany)
 * and is private by copyright law - it is NOT Freeware.
 *
 * Any unauthorized use of this software without a valid license
 * is a violation of the license agreement and will be prosecuted by
 * civil and criminal law.
 *
 * @copyright  2018 Lifestyle Webconsulting GmbH
 * @link       http://www.life-style.de
 */

namespace Sso\RestBundle\ModelXml\Request\UserWs\Version2\UserApplication\ShowDetails;

use Sso\RestBundle\ModelXml\Common\Request\Credentials\UserWs;

/**
 * Class Factory
 * @package Sso\RestBundle\ModelXml\Request\UserWs\Version2\UserApplication\ShowDetails
 */
class Factory
{
    /**
     * @var array
     */
    protected $facSetter;

    /**
     * @return Request
     */
    public function request()
    {
        if (!isset($this->facSetter[__FUNCTION__])) {
            $this->facSetter[__FUNCTION__] = new Request();
        }
        return $this->facSetter[__FUNCTION__];
    }

    /**
     * @return UserWs\Credentials
     */
    public function credentials()
    {
        if (!isset($this->facSetter[__FUNCTION__])) {
            $this->facSetter[__FUNCTION__] = new UserWs\Credentials();
        }
        return $this->facSetter[__FUNCTION__];
    }

    /**
     * @return UserApplication
     */
    public function userApplication()
    {
        if (!isset($this->facSetter[__FUNCTION__])) {
            $this->facSetter[__FUNCTION__] = new UserApplication();
        }
        return $this->facSetter[__FUNCTION__];
    }

    /**
     * @return ShowDetails
     */
    public function showDetails()
    {
        if (!isset($this->facSetter[__FUNCTION__])) {
            $this->facSetter[__FUNCTION__] = new ShowDetails();
        }
        return $this->facSetter[__FUNCTION__];
    }

    /**
     * @return Key
     */
    public function key()
    {
        if (!isset($this->facSetter[__FUNCTION__])) {
            $this->facSetter[__FUNCTION__] = new Key();
        }
        return $this->facSetter[__FUNCTION__];
    }

    /**
     * @return UserType
     */
    public function userType()
    {
        if (!isset($this->facSetter[__FUNCTION__])) {
            $this->facSetter[__FUNCTION__] = new UserType();
        }
        return $this->facSetter[__FUNCTION__];
    }

    /**
     * @return ApplicationType
     */
    public function applicationType()
    {
        if (!isset($this->facSetter[__FUNCTION__])) {
            $this->facSetter[__FUNCTION__] = new ApplicationType();
        }
        return $this->facSetter[__FUNCTION__];
    }
}
